# rtic_bare1

What it covers:

- Compilation and Debugging.
- Arithmetics and `panic`s.

---

## Setup

Make sure that your repository is updated (pull from upstream).
Connect you nRF52840 DK with a micro USB connector to your host machine.

---

## Exercise 1

- Click on the [Run and Debug] button (or press Ctrl-Shift-D)
- Choose the "probe-rs Debug" profile in the dropdown.
- Click on the Start Debugging arrow (or press F5).

This compiles the chosen example in dev/debug mode and and starts a debug session.

The program will be flashed and run. You can look at the `.vscode/launch.json` for configuration options.

Under the RTIC will generate code that calls `init`, and ends up in `idle` (as no tasks are defined.)

You should have encountered a Rust panic.

Paste the error message:

```
panicked at examples/rtic_bare1.rs:43:13:
attempt to add with overflow
```

Look at the source code and explain in your own words why the code panic:ed.

```
x is set to max u32. Then we try to add to i, which is referencign x, causing an overflow within the calcualtion.
```

Commit your answer (bare1_1)

---

## Exercise 2

Now let's try to break it down to see what caused the panic.

Put a breakpoint at `*i += 1;` (line `(A)`). Click to the left of the line marker, you get a red dot.

Restart the debug session (Ctrl-Shift F5), the program should now be halted (paused) at line '(A)'.

What is the value of `i`?

```
i = 0x2003FFD4
```

What does `i` refer to? (Where does it point?)

```
i refrences the mutable variable  x
```

Which variable is owning the value that `i` has mutably borrowed?

```
x
```

What is the value of `*i`? (You need to expand the `Variables` tab.)

```
i = 4294967294
```

Now continue the program (F5), since you are in a loop the program will halt again at line '(A)'.

What is the value of `*i`?

```
i = 4294967295
```

Now continue again.

At this point your code should `panic` (there should be a panic message in the terminal). The program is now stuck in an infinite loop in the `panic_rtt_target` library. Press `pause` (F6) to halt the processor. Doing so will show the CALL STACK frames.

!!! THIS SEEMS TO BE CURRENTLY BROKEN     !!!
!!! BUT IT SHOULD WORK AS DESCRIBED BELOW !!!

```text
panic @0x00001c38 
increment @0x00000330 
idle @0x0000035a 
main @0x00000392 
```

You can navigate the CALL STACK, select the `increment` frame, you should see line `(A)` being highlighted, i.e. the line of code that was executed causing the panic to occur.

Now, select the `idle` frame.

What is the number of the highlighted line?

```
37
```

Explain in your own words why this makes sense.

```
This line causes the overflow.
```

What is the value of the variable `x`?

```
x = 4294967295
```

Explain in your own words why this value makes sense.
(Hint, confer to the value of `*i` in the `increment` frame).

```
*i = 4294967295 in the increment frame. Then when we add to it we go above the max u32 value and the program panics as it should. Therefore we dont increase the size of *i
```

Commit your answer (bare1_2)

---

## Exercise 3

Now lets have a look at the generated assembly.

First restart the debug session, and the program should break at line `(A)`.

Right click to get a drop down and select `Open Disassembly View`.

In the `Disassembly` you will see one assembly line highlighted, corresponding to the `Program Counter` (PC) register value (the assembly instruction being executed when halted).

You can see all register values in the `Register` pane to the left.

What value is shown for the PC?

```
R15/PC = 0x00000238 (but I dont understand why, I should be at 0x00000232??? In the incremental function)
```

Now look at the `Disassembly` what is the address of the highlighted line?

```
0x000001FA (But I should be on like 0x00000232 or within the incremental function???)
```

The assembly code here was generated by Rust implementing the `increment` function. (You will find `fn increment ....`, a few lines above the halted instruction.)

See [Thumb instruction set summary](https://developer.arm.com/documentation/ddi0210/c/Introduction/Instruction-set-summary/Thumb-instruction-summary) for further details.

The code you see is not optimized (Development/Debug profile), and quite ineffective/lengthy.

Optional, look at the generated assembly, identify the function entry code (stack push), the addition by one, the check of overflow parts of the generated assembly code.

[Your optional answer here, explain the generated assembly code in your own words]

Commit your answer (bare1_3)

---

## Exercise 4

So far we have considered overflows as errors. In cases we want modular/wrapping arithmetics. Using Rust you have the option to choose. We can select type of arithmetics either globally, per variable, or even per operation.

Let's try the global option first. In the `Cargo.toml` file you find.

``` toml
[profile.dev]
incremental = false
codegen-units = 1
lto = true
# overflow-checks = false # uncomment to disable overflow checks for dev/debug builds  
```

Uncomment the `overflow-checks = false` and re-run the experiment using the `probe-rs Debug` profile, with a breakpoint set at line '(A)'.

You program should halt at `(A)`. What is the value for `*i`?

```
i = 4294967294
```

Continue (F5). What is the new value of `*i`?

```
i = 4294967295
```

Continue (F5). What is the new value of `*i`?

```
i = 0
```

Hint: At this point `*i`, should now have wrapped around to 0 as we disabled the overflow checking the addition is now evaluated under *modular* arithmetics.

Now let us see how what Rust will do when allowed to optimize.

``` toml
[profile.release]
incremental = false
codegen-units = 1   # better optimizations
debug = true        # symbols are nice and they don't increase the size on Flash
lto = true          # better optimizations
# overflow-checks = true # uncomment (or set to false) to enable overflow checks for release builds
```

Overflow checking is (for performance reasons) disabled for *release* builds. Select the `probe-rs Release` profile and re-run the experiment.

In this case you will find that the breakpoint is greyed out, so we have to press pause (F6) to halt.

Rust + LLVM does a very good job at optimization but a lot of debug information is lost on the way. Thus it is hard for the debugger to give a good debug experience for highly optimized code.

What we can do in this case is to look at the PC register value (0x00000160, in my particular build case, but may vary depending on toolchain version).

In the `Disassembly` view locate this particular address.

You should find something like:

``` asm
0x00000160 b 0x160
```

This instruction performs an *unconditional* branch to itself, thus infinitely looping, without touching the `*i`.

At a first glance this seems wrong! But did the compiler really make a mistake here?

Recall the Rust "aliasing" rules, a resource (value) may only be accessed by a single unique mutable reference. Thus the Rust compiler knows that there is just a single owner of the value (the variable `x` in `idle`). The `increment` function is non returning, thus `idle` will never be resumed. In effect, the value `*i` is not observed/used anywhere besides in the infinite loop:

``` rust
loop {
   *i += 1;
}
```

And as we build in release mode with overflow checking disabled, no error can occur in the addition (hence, the operation is free of side-effects). Based on the non-observability of the value, and that the operation is free of side-effects, Rust is comes to the *correct* conclusion that the add operation can be optimized out.

Thus translating `loop { *i += 1; }` into `0x00000160 b 0x160` is perfectly Ok (and the best that can be done).

Commit your answer (bare1_4)

---

## Exercise 5

Now let's look into how we can choose type of arithmetic operation at per variable.

Rust supports a rich and powerful type system. One example is the so called *newtype* pattern, allowing us to associate semantics (meaning) to operations of the given type. A prominent example is [Wrapping](https://doc.rust-lang.org/stable/core/num/struct.Wrapping.html).

We can use this to give wrapping (modular) semantics to our `x` variable and function argument.

``` rust
   let mut x = Wrapping(core::u32::MAX - 1);
```

Notice the type of the argument is a mutable reference to `Wrapping<u32>`, where `u32` is a type argument.

``` rust
   pub fn increment(i: &mut Wrapping<u32>) -> ! {
      loop {
         *i += 1;
      }
   }
```

Make those changes to the code and store this under the file name `rtic_bare1_wrapping.rs`.

To test that this works, first we restore the `Cargo.toml` to its initial configuration, which is set to use checked arithmetics.

``` toml
[profile.dev]
...
# overflow-checks = false # uncomment to disable overflow checks for dev/debug builds  

[profile.release]
...
overflow-checks = true # uncomment (or set to false) to enable overflow checks for release builds
```

Now run the code (`rtic_bare1_wrapping`) using the `probe-rs Debug` profile, with a breakpoint set at line `(A)`.

Inspect the value of the `i` variable.

What is the vale/ type shown for `i` under `Variables`?

```
&mut core::num::wrapping::Wrapping<u32> @ 0x2003FFD4
```

What is the value / type shown for `*i`?

```
Wrapping<u32> @ 0x2003FFE0
```

What is the value / type shown if you expand `*i`?

```
4294967294
```

(Rust is treating a *newtype* as a single field 'struct' with the field name `__0:`.)

Now continue (F5).

What is the new expanded value for `*i`.

```
4294967294
```

Now continue (F5).

What is the new expanded value for `*i`.

```
4294967295
```

Now continue (F5).

What is the new expanded value for `*i`.

```
0
```

Hint: You should have obtained a 0 here, the value should wrap for this SPECIFIC variable (and mutable references thereof).

Now stop the debug session and choose the `probe-rs Release` profile.

Run the application and press pause to stop it.

What is the value of the PC.

```
R15/PC = 0x00000158
```

What is the assembly instruction at this address?

```
b #0x158
```

Add the `rtic_bare1_wrapping.rs` to your git repo and commit your answer (bare1_5)

---

## Learning Outcomes

- Compilation and Debugging of Rust applications on embedded targets.

- Embedded code typically is performance sensitive, hence it is important to understand how code is generated to achieve efficient and correct implementations.

- Arithmetics are key to the processing of data, so its important that we are in control over the computations. E.g. computing checksums, hashes, cryptos etc., all require precise control over wrapping vs. overflow behavior.

- In the `Cargo.toml` file we define `profiles` for development/debug builds and release mode builds. For each type of build we can select the wrapping/overflow behavior. If you write a library make sure the code works both in debug/dev and release mode. 

- For per-variable, control we can use the `Wrapping` newtype pattern (giving us modular arithmetics).

  Even finer grained control over arithmetic operations is possible using explicit operations, see e.g. [u32](https://doc.rust-lang.org/core/primitive.u32.html), there you find [checked_add](https://doc.rust-lang.org/core/primitive.u32.html#method.checked_add), [saturating_add](https://doc.rust-lang.org/core/primitive.u32.html#method.saturating_add), [unchecked_add](https://doc.rust-lang.org/core/primitive.u32.html#method.unchecked_add) etc. These are fully portable, so they work the same even if the underlying hardware treats integers differently from each other.

- Arithmetics in Rust are fully portable, they work the same even if the underlying hardware treats integers differently from each other. (This is not the case for C/C++, where behavior will vary depending on the HW.)

You may return to this exercise to recap how you can do fine grained debugging if run into problems later.
