# rtic_bare3

What it covers

- Reading Rust documentation
- Timing abstractions and semantics
- Understanding Rust abstractions

 ---

## Setup

``` shell
cargo doc --example rtic_bare3 --open
```

(When documenting make sure that only one panic handler is enabled in Cargo.toml.)

In the docs, search (`S`) for `Monotonic` and read the API docs. Also search for `Instant`, and `Duration`. You will later use the `now` method, giving you an `Instant` in time. The difference between two `Instant`s is a `Duration`. (There are some ugly looking type parameters, but we can skip those for now...) 

To summarize:

- `Monotonic` is a "trait" for a timer implementation.
- `Instant` is a point in time.
- `Duration` is a range in time.

The `Systick::new(systick, FREQ_CORE)` returns a timer implementing the `Monotonic` trait. The `systick` *peripheral* is moved into the constructor, i.e.,the peripheral will be *consumed*. Thus, you cannot by mistake construct two timers from the same peripheral. The `FREQ_CORE` parameter is set to the core clock frequency (allowing to convert ticks into wall clock time). 

Using a timing library, e.g.[fugit](hhttps://docs.rs/fugit/latest/fugit/index.html), allows you operate on and convert between different time representations.

---

## Exercise 1

Build and run the application in vscode using `probe-rs Debug`.

What is the output in the Terminal?

```
--- init ---
Start Instant { ticks: 0 }
End Instant { ticks: 175 }
Diff Duration { ticks: 1750 }
Diff 1750 ms
```

Now, build and run the application in vscode using `probe-rs Release`.

What is the output in the Terminal?

```
--- init ---
Start Instant { ticks: 0 }
End Instant { ticks: 10 }
Diff Duration { ticks: 100 }
Diff 100 ms
```

Compute the speedup (how many times faster is release)

```
1750 / 100 = 17.5  So 17.5 times faster
```

Does this correspond to the speedup obtained in `rtic_bare2` (if not exact think about why the speedup may differ?)

Hint: What about timing precision for the `mono` timer used in this example.

```
This example was faster than in bare2 for me. As the code is otherwise the same, I'm guessing it boils
down to this implementation of time meassurement being more efficient. The Systick implementation is
based on interrupts, while the monotonic implementation is hardware based. The Systick implementation 
has a lower granularity, only looking at timeing, while the hardware-based solution can look at accurate
cycle timings, which would be more demanding performance wise. So thats partly from where the speedup
comes from.
```

Commit your answers (bare3_1)

---

## Exercise 2

The implementation of `Systick` as a *monotonic* timer sets up periodic interrupt, in this case with a period of 10 milliseconds.

``` rust
type MyMono = Systick<100>; // 100 Hz, 10 ms granularity
```

On each interrupt the monotonic timer is increased. In cases you might want finer granularity (10ms is a fairly long time.)

Change the timer granularity to 10 micro seconds.

``` rust
type MyMono = Systick<100_000>; // 100 000 Hz, 100 kHz, 10 us granularity
```

Build and run the application in vscode using `probe-rs Release`.

What is the output in the Terminal?

```
--- init ---
Start Instant { ticks: 0 }
End Instant { ticks: 12779 }
Diff Duration { ticks: 127 }
Diff 127 ms
```

Explain in your own words why the measured value in milliseconds differs from Exercise 1 above.

Hint: What about the overhead occurring from the `Systick` timer interrupt.

```
As the time interval between the systick interrupts are smaller, we execute more of them during the program.
This costs us some extra instructions each time, and therefor the performance result is worse.
```

Commit your answers (bare3_2)

---

## Exercise 3

In `examples/rtic_bare3_dwt`, you find an alternative implementation of the same program that uses the `DwtSystick` *monotonic* implementation. In contrast to the `Systick` "tick" based implementation `DwtSystick` uses the DWT cycle counter as a free-running timer.

This allows cycle accurate timing, while significantly reducing the number of interrupts the processor has to manage (and thus reduce CPU load).

Build and run the application in vscode using `probe-rs Release`.

What is the output in the Terminal?

```
--- init ---
Start Instant { ticks: 40 }
End Instant { ticks: 4000071 }
Diff Duration { ticks: 62 }
Diff 62 ms
```

Explain in your own words why the measured value in milliseconds relates to Exercise 1 and 2 above.

```
I'm not sure what is exactly meanat by "measure value in miliseconds relates", but the time with DWT is shorter.
I think this is because DWT is not really a system timer, but rather a cycle timer. So what we do instead is
that we check for the cycles and compute a time, rather then check the time while running the code. This helps
to achieve less overhead, and we therefore get a faster result.
```

Commit your answers (bare3_3)

---

## Learning Outcomes

- Time can be treated in a platform independent manner:

  In this exercise we have shown that we can step away from pure hardware accesses and deal with time in a more convenient and "abstract" fashion.

  `Instant` and `Duration` are associated with semantics (meaning) for working with time in an abstract way.

  - `Monotonic` as a trait is defines an API (e.g, the `now` method), and
  - `Systick` provides an implementation working for **all** Cortex M processors. 
  - `DwtSystick` provides an implementation that works for all Cortex M processors supporting a `DWT` unit (`DWT` is not implemented for the smallest Cortex M0/M0+ cores).

  If you implement your application based on `Instant` and `Duration`, your code will be "portable" across all platforms given that the platform provides a timer implementing the `Monotonic` trait (e.g., `Systick` or `DwtSystick`).

- The `Monotonic` trait just defines the API for a *monotonic* timer. E.g., we can have a Real-Time clock (with very low drift) for calendar like functionality, and at the same time another monotonic timer that provide fine granularity but shorter range.

- Code re-use:

  The implementation (testing and verification) of a `Monotonic` timer needs to be done only once, not scattered across thousands of manually written applications.

- Tradeoff between precision and overhead:

  The `Systick` implementation is "tick" based and allows to trade-off precision (granularity) to system load (code running to each time the `Systick` ("tick") interrupt is triggered). Other implementations of `Systick` may use a free running timer instead of the "tick" based approach, in this way an interrupt is triggered *only* when the full range of the timer is reached, thus significantly reducing the overhead. An extreme is the use of an hardware Real-Time-Clock (RTC), which has an in practice infinite range (thus infers no interrupt overhead).

  As a comparison, the Linux kernel is driven by a system tick (jiffy). By default it is set to 10ms, as a reasonable tradeoff between timing accuracy and interrupt handling overhead.
