# rtic_bare4

What it covers:

- Raw pointers
- Volatile read/write
- GPIO (a primitive abstraction)
- Bitwise operations

---

## Background

Hardware access is at the heart of embedded programming. In this exercise we will take a very primitive and unsafe approach to accessing peripherals.

We will use raw pointer reading/writing similar to C/C++ style programming. Later you will learn how to make safe abstractions to underlying hardware.

---

## Exercise 1

Build and run the application use the debug profile (`probe-rs Debug`).

Did you enjoy the blinking?

```
Many nice blink, yesyes
```
:sunglasses:

Now lookup the [nRF52840_PS_v1.8](https://infocenter.nordicsemi.com/pdf/nRF52840_PS_v1.8.pdf), and read each section referred:

- 6.9   GPIO - General Purpose input/output
- 6.9.2 Registers

Also look in the documentation for the [nRF52840 DK](https://infocenter.nordicsemi.com/pdf/nRF52840_DK_User_Guide_20201203.pdf).

Search for `LED 1` and figure out how it is connected.

Document the program:

- Explain at a higher level what the program does, what is the purpose of lines marked `A`, `B` and `C`. You may want to put a breakpoint at each `wait` and run let the program break at these to inspect the state of the LED.

- For each low-level access `A`, `B` and `C` refer to the corresponding section in the Product Specification (data-sheet), and the dev-kit manual.

- Explain the address computation in `A`, and bitwise operations in `A`, `B` and `C`. (What are the effects.)

``` Rust
write_u32(P0_CNF + 13 * 4, 1 << 0); 
// A: Configure GPIO pin 13, which controls LED1. 
// In the first "field" we compute the adress of the configuratio register for GPIO pin 13.
// In the 2nd "field" we perform the bitwise operation of setting the registers LSB to 1 high.

write_u32(P0_SET, 1 << 13); // B: Here we SET the GPIO pin 13 to high. This turns on the LED1. Bitwise operation in the register to shift by one

write_u32(P0_CLR, 1 << 13); // C: Here we CLEAR the GPIO pin 13, we set it to low. This turn off the LED1. Bitwise operation in the register to shift by one

// The pin numbering and access can be read about on page 155 in the Product Specification, and on page 29 in the Dev Kit Manual.
```

Commit your answers (bare4_1)

---

## Exercise 2

The function `write_u32` is marked `unsafe`.

Explain in your own words why?

```
Because we are directly handling a critical register, which is by definition 
an unsafe operation. If this code could be influenced from external sources,
this is the stuff that makes system go boom boom, no saftey ;)
```

The function `read_u32` is also marked `unsafe`.

Explain in your own words why?

Hint: Read the documentation [read_volatile](https://doc.rust-lang.org/core/ptr/fn.read_volatile.html). Even reading a memory address might have side effects...

```
Here the problem lies in the "validity" of the information. If we read a register without any protection,
 the value can be changed between when we read it and to when we might actually do something with it.
```

Commit your answers (bare4_2)

---

## Exercise 3

Volatile read/writes are explicit *volatile operations* in Rust, while in C they are declared at type level (i.e., access to variables declared volatile amounts to volatile reads/and writes).

Both C and Rust (even more) allows code optimization to re-order operations as long as data dependencies are preserved.

What would happen if `B` is ordered after `C`.

```
The starting order off teh light being on/off changes to straing while off. Then it continues as normal as they just swtitch
```

Notice `B` and `C` refer to different locations/addresses and has as such no data dependencies. Thus, without the ordering property of volatile operations Rust would have been free to re-order them. (Re-ordering is done by the compiler to improve the efficiency of the code.)

Commit your answers (bare4_3)

---

## Exercise 4

Now uncomment the two lines marked `D`, and run the code.

What are the printed values of `P0_DIR`?

```
P0_DIR 0
P0_DIR 8192
```

Explain in your own words, how the value of the `>P0_DIR` register is changed without any operation writing to it.

```
When we do changes for P0_CNF after the first print, the value changes for _DIR. After configuring P0,
we then read it again and find the value of 8192, which is the value from DIR.
```

Hint: Read the documentation for the `DIR` and `CNF` registers carefully, the answer is there.

Commit your answers (bare4_4)

---

## Exercise 5

The vscode plugin also supports parsing and displaying System View Description (SVD) files showing the memory mapped peripherals. In the `launch.json` you give a path to the SVD file:

``` toml
"coreConfigs": [
    {
        "coreIndex": 0,
        ...
        "svdFile": "./.vscode/nrf52840.svd"
    }
]
```

Under `VARIABLES/Peripherals`, you find the complete list of peripherals. 

Navigate to the `GPIO` peripheral, expand it find and `Add to Watch` both `GPIO.P0.OUT.PIN13` and `GPIO.P0.PIN_CNF13`.

Restart the application (it should halt at the first `wait`. At this point you should now see the values of `GPIO.P0.OUT.PIN13` and `GPIO.P0.PIN_CNF13`. You can expand the `GPIO.P0.PIN_CNF13` to find all the fields.

What are the value of each field in `CNF13`

```
GPIO.P0.PIN_CNF13.DIR: 1 @ 0x50000734:0..1
GPIO.P0.PIN_CNF13.INPUT: 0 @ 0x50000734:1..2
GPIO.P0.PIN_CNF13.PULL: 00 @ 0x50000734:2..4
GPIO.P0.PIN_CNF13.DRIVE: 000 @ 0x50000734:8..11
GPIO.P0.PIN_CNF13.SENSE: 00 @ 0x50000734:16..18
```

Commit your answers (bare4_5)

---

## Learning Outcomes

- You have learned about `volatile` read/write operations and the importance of instruction ordering.

- You have introduced a primitive abstraction to reading/writing raw data, and abstract their addresses using constants (similar to C). 

- Similar to C, our interface (API) is still completely unsafe. (We can pass in any address to the `read/write` functions, thus correctness is completely up to the programmer.)

- As you have seen in exercise 4, you really need to understand the underlying hardware by reading manuals and data sheets to understand what is happening. The devil is in the details.

- In exercise 5 you learned how to use vendor provided SVD files in order to watch peripheral registers changing over time. This can be very useful when developing and debugging low-level code.
