# rtic_bare5

What it covers:

- abstractions in Rust
- `struct` and `impl`
- bitwise operations

---

## Background

In `rtic_bare4` we accessed the GPIO peripheral using unsafe code (the `read_u32/write_u32` took any arbitrary address as argument) so correctness was up to the programmer.

Rust allows allows us to build (zero-cost) abstractions, where the expert (i.e., You) can handle the dangerous parts and provide a fail safe API to the users of your libraries.

In this exercise we have model register access through `VolatileCell<T>`, which uses unsafe volatile read/write operations. `T` here can be any type that implements `Copy` (e.g., our primitive types, u8/u32, etc.).

We use `VolatileCell` to model the `GPIO`, in effect `GPIO_P0/P1` can be seen as _overlaying_ the hardware GPIO ports P0/P1 using the `GPIO` struct.

In this example we construct the peripheral abstraction by hand. Tools like [svd2rust](https://github.com/rust-embedded/svd2rust) automatically constructs Peripheral Access Crates (PACs) from vendor's hardware descriptions (SVD files), but the principle is similar.

For simplicity the modules `raw_access` and `nrf52840` are in this example part of the user application. Normally you would factor out these in a library (for you and others to re-use).

Commit your answer (bare5_1)

---

## Exercise 1

First run the example just to confirm that you got the led blinking.

Did you enjoy the blinking?

```
yesyes very blink
```

Inspecting the user code `app`, you will find that only a single line using unsafe code.

Explain in your own words why `get` requires `unsafe`.

Hint: It is marked `unsafe` for a reason (confer to the Rust aliasing rules).

```
Because it is a raw pointer dereference, which is unsafe.
```

`{:p}` allows you to print an address in a nicely formatted way. What is the address printed for `gpio_p0.PIN_CNF[13]`?

```
address PIN_CNF[13] 0x50000734
```

Check in the product specification [nRF52840_PS_v1.7](https://infocenter.nordicsemi.com/pdf/nRF52840_PS_v1.7.pdf) section 6.9.2 what this address represents. (No surprises there right:)

We see that `get` returns a raw pointer to this `GPIO` struct with the address computed as follows:

```rust
impl GPIO_P0 {
   pub fn get() -> *mut GPIO {
      (0x5000_0000 + 0x504) as *mut GPIO
   }
}
```

Now look at the `GPIO` struct definition. It is marked with the attribute:

```rust
 #[repr(C)]
```

This forbids Rust to do any smart padding/alignment and reordering of fields (what you C is what you get). This allows us to _overlay_ a structure on-top of the underlying HW. Explain in your own words the purpose of `_PAD1` in the `GPIO` struct and why it is not declared as `pub`.

```
Well it is padding right? So its supposed to do nothing but block space so that rust dosent do anything to agressive while optimizing.
```

Explain in your own words why `[u8; 0x700-0x528]` gives us the right address for the following `PIN_CFN`.

```
It is the correct amount of space to keep PIN_CNF alligned with the hardware registers.
```

Explain in your own words how `PIN_CNF[13]` gives us the right address for accessing pin configuration register 13. (You may look at `rtic_bare4.rs` to compare how the address was computed in a more primitive manner.)

```
The parts of the calculation is: Base address + padding + index for pin 13. 
```

Commit your answer (bare5_1)

---

## Exercise 2

Hardware registers are typically divided into (bit) fields. It is quite common that you want to operate on a certain field. This can be accomplished by first reading the register, operate on the read value (and/or) to set the bits of interest, and finally write the result back to the register.

Doing it so manually is error prone, and we would like a "fail safe" way of doing this. Let us extend the API of `VolatileCell` to allow for _modifying_ fields of an underlying `u32` register.

`modify` takes 4 parameters:

- `&mut self`,
- `offset: u8`, the field offset in bits,
- `width: u8`, the field to width in bits, and
- `value: u32`, the value of the field.

E.g., a call:

```rust
t.modify(5, 3, 0b1110101);
```

Implies that bits 7,6,5 of `t` should be updated to hold `101` (the three lowest bits of the `value` passed). Other bits of the `value` argument should be ignored. Other bits in `t` should remain their value.

_Alternatively_, values ranging outside of width of the field to update could be considered illegal. In this case the above `modify` call would result in a `panic`. If you choose to implement this alternative API you need to change the provided test accordingly.

A skeleton implementation of `modify` is provided:

```rust
impl VolatileCell<u32> {
   #[inline(always)]
   pub fn modify(&mut self, offset: u8, width: u8, value: u32) {
      let r = self.read();
      rprintln!("r = {:b}", r);

      rprintln!("value = {:b}", value);
      let or_mask = 0; // replace with your own code
      rprintln!("or_mask {:b}", or_mask);
      let and_mask = 0; // replace with your own code
      rprintln!("and_mask {:b}", and_mask);

      let new_value = (r & and_mask) | or_mask;

      self.write(new_value);
   }
}
```

Add your own code for setting the `or_mask` and the `and_mask` (instead of the 0).

To test that your implementation works as expected uncomment line `D`, in `idle`:

```rust
raw_access::test_modify(); // D
```

You should leave the tracing printouts for review purpose (a realistic implementation would of course omit the tracing the intermediate masks).

Commit your answer (bare5_2)

--Notice--

As with arithmetic operations, default semantics differ in between debug/dev and release builds. In debug << rhs is checked, rhs must be less than 32 (for 32 bit datatypes).

Over-shifting (where bits are spilled) is always considered legal, it is just the shift amount that is checked.

There are explicit unchecked versions available if so wanted.

---

## Learning Outcomes

- How to _overlay_ underlying memory mapped I/O by Rust data structures.
- The use of abstractions to reduce the need for `unsafe` user code.
- How to implement your own _"fail safe"_ API for low-level bit masking.

The Rust embedded echo system relies on the principles you have learned here.

The `cortex-m` crate for ARM Cortex-M peripherals, vendor specific Peripheral Access Crates (PACs), and Hardware Abstraction Layers (HALs), to name a few.

In the next set of labs you will learn how to make use of existing libraries, allowing us to go from zero to embedded hero in just 10 weeks!!!!
