//! rtic_bare8b.rs
//!
//! What it covers:
//! - serial communication
//! - good design
//!

#![no_main]
#![no_std]

use panic_rtt_target as _;


#[rtic::app(device = nrf52840_hal::pac, dispatchers = [TIMER0])]
mod app {

    use {
        embedded_hal::digital::v2::OutputPin,
        command_parser::{parse, parse_result, Command},
        cortex_m::asm,
        hal::{
            gpio::{Level, Output, Pin, PushPull},
            pac::UARTE0,
            prelude::*,
            uarte::{self, UarteRx, UarteTx},
        },
        fugit::{MillisDurationU64, TimerInstantU64},
        nb::block,
        nrf52840_hal as hal,
        rtt_target::{rprintln, rtt_init_print},
        systick_monotonic::*,


    };

    pub fn led_on(_led: &mut impl OutputPin) {
        let _ = _led.set_low();
    }

    #[inline(never)]
    pub fn led_off(_led: &mut impl OutputPin) {
        //rprintln!("led_off");
        let _ = _led.set_high();

    }

    const TIMER_HZ: u32 = 4; // 4 Hz (250 ms granularity)
    const TIME_0: Instant = TimerInstantU64::from_ticks(0); // Constant for time zero

    type Led = Pin<Output<PushPull>>;
    type Instant = TimerInstantU64<TIMER_HZ>;
    type Duration = MillisDurationU64;

    #[monotonic(binds = SysTick, default = true)]
    type MyMono = Systick<TIMER_HZ>;


    #[shared]
    struct Shared {
        freq: u32,
        duty: u8,
        run: bool,
    }


    #[local]
    struct Local {
        tx: UarteTx<UARTE0>,
        rx: UarteRx<UARTE0>,
        led: Led,
    }

    #[init(local = [
        SERIAL0_TX_BUF: [u8; 1] = [0; 1],
        SERIAL0_RX_BUF: [u8; 1] = [0; 1],
    ])]
    fn init(cx: init::Context) -> (Shared, Local, init::Monotonics) {
        rtt_init_print!();
        rprintln!("init");

        let device = cx.device;
        let mut mono = Systick::new(cx.core.SYST, 64_000_000);
        let p0 = hal::gpio::p0::Parts::new(device.P0);

        // enable UARTE0 endrx interrupt
        device.UARTE0.intenset.modify(|_, w| w.endrx().set_bit());

        let (tx, mut rx) = uarte::Uarte::new(
            device.UARTE0,
            uarte::Pins {
                txd: p0.p0_06.into_push_pull_output(Level::High).degrade(),
                rxd: p0.p0_08.into_floating_input().degrade(),
                cts: None,
                rts: None,
            },
            uarte::Parity::EXCLUDED,
            uarte::Baudrate::BAUD115200,
        )
        .split(cx.local.SERIAL0_TX_BUF, cx.local.SERIAL0_RX_BUF)
        .expect("Could not split serial0");

        let next_instant = mono.now() + 1.secs();
        blink::spawn_at(next_instant, next_instant).unwrap();


        // on NRF* serial interrupts are only called after the first read
        let _ = rx.read();

        let _led = p0.p0_13.into_push_pull_output(Level::Low).degrade();

        let freq = 2; // 2 Hz
        let duty = 50; // 50% duty cycle

        (Shared {

            freq,
            duty,
            run: true,

        }, Local { tx, rx, led: _led }, init::Monotonics(mono))
    }

    #[idle()]
    fn idle(_cx: idle::Context) -> ! {
        rprintln!("idle");

        loop {
            //rprintln!("sleep");
            asm::wfi();
            //rprintln!("wake");
        }
    }

    #[task(capacity = 10, shared=[freq, duty, run], local = [tx, len: usize = 0, data_arr :[u8; 10] = [0; 10]])]
    fn data_in(mut cx: data_in::Context, data: u8) {
        let tx = cx.local.tx;
        let len = cx.local.len;
        let data_arr = cx.local.data_arr;

        //rprintln!("data_in {}", data);
        block!(tx.write(data)).unwrap();
        match data {
            13 => {
                let slice = &data_arr[0..*len];
                rprintln!("return {:?}, str {:?}", slice, core::str::from_utf8(slice)); 
                /*rprintln!(
                    " {:?}... parse {:?}, parse_result {:?}",
                    slice,
                    parse(slice),
                    parse_result(slice)
                ); */


                *len = 0;
    

                cx.shared.duty.lock(|duty| match parse_result(slice) {
                    
                    Ok(Command::Duty(d)) => {
                        rprintln!("duty set with {:?}", d);
                        *duty = d;
                    }

                    Ok(Command::FrequencyHz(f)) => {
                        rprintln!("freq set with {:?}", f);
                        cx.shared.freq.lock(|freq| {
                            *freq = f;
                        });

                    }
                    Ok(Command::Start) => {
                        rprintln!("start");
                        cx.shared.run.lock(|run| {
                            *run = true;
                        });

                    }
                    Ok(Command::Stop) => {
                        rprintln!("stop");
                        cx.shared.run.lock(|run| {
                            *run = false;
                        });

                    }
                    _ => {
                        rprintln!("error");
                    }
                })

                // empty the data array
                

            }
            _ => {
                data_arr[*len] = data;
                //*len += 1;
                *len = usize::min(*len + 1, data_arr.len() - 1);
            }
        }
    }


    #[task(shared=[freq,duty,run],local = [cnt: u32 = 0, led])]
    fn blink(mut cx: blink::Context, instant: Instant) {
        let mut offset: u64 = 0;
        cx.shared.freq.lock(|frq| {
            cx.shared.duty.lock(|dty| {

                let T_ms = 1000.0 / *frq as f64;
                let on_ms = T_ms * (*dty as f64 / 100.0);
                let off_ms = T_ms - on_ms;

                cx.shared.run.lock(|run| {
                    if *run {
                        if *cx.local.cnt % 2 == 0 {
                            led_on(cx.local.led);
                            offset = on_ms as u64;
                        } else {
                            led_off(cx.local.led);
                            offset = off_ms as u64;
                        }
                    } else {
                        led_off(cx.local.led);
                        offset = T_ms as u64;
                    }

                    /*rprintln!("T_ms{:?}", T_ms);
                    rprintln!("on_ms : {:?}", on_ms);
                    rprintln!("off _ms {:?}", off_ms);
                    rprintln!("offset {:?}", offset);*/
                });
            });
        });

        *cx.local.cnt += 1;
        let next_instant = instant + offset.millis();
        blink::spawn_at(next_instant, next_instant).unwrap();
   
    }


    #[task(binds = UARTE0_UART0, priority = 2, local = [rx])]
    fn uarte0_interrupt(cx: uarte0_interrupt::Context) {
        rprintln!("uarte0 interrupt");

        while match cx.local.rx.read() {
            Ok(b) => {
                rprintln!("Byte on serial0: {}", b);
                data_in::spawn(b).unwrap(); // panic if queue full
                true
            }
            Err(err) => {
                rprintln!("error {:?}", err);
                false
            }
        } {
            // just continue with next read
        }
    }
}

