//! examples/rtc_usb_serial
//!
//! CDC-ACM serial port example using polling in a busy loop.S
#![no_std]
#![no_main]

use panic_rtt_target as _;

#[rtic::app(device = nrf52840_hal::pac)]
mod app {

    use {
        hal::clocks::*,
        hal::usbd::{UsbPeripheral, Usbd},
        nrf52840_hal as hal,
        rtt_target::{rprintln, rtt_init_print},
        usb_device::{
            prelude::*,
            class_prelude::UsbBusAllocator,
            device::{UsbDeviceBuilder, UsbVidPid},
        },
        usbd_serial::{SerialPort, USB_CLASS_CDC},
        command_parser::*,
    };


    #[shared]
    struct Shared {}

    #[local]
    struct Local {
        usb_dev: UsbDevice<'static, Usbd<UsbPeripheral<'static>>>,
        serial: SerialPort<'static, Usbd<UsbPeripheral<'static>>>,
    }
    

    #[init(local = [
        clocks: Option<Clocks<ExternalOscillator, Internal, LfOscStopped>> = None,
        usb_bus: Option<UsbBusAllocator<Usbd<UsbPeripheral<'static>>>> = None, 
    ])]


    fn init(cx: init::Context) -> (Shared, Local, init::Monotonics) {
        rtt_init_print!();
        rprintln!("\n--- init ---");

        //Enable USBD interrupt
        cx.device.USBD.intenset.write(|w| w.sof().set());

        let periph = cx.device;
        let clocks = Clocks::new(periph.CLOCK);

        // make static lifetime for clocks
        cx.local.clocks.replace(clocks.enable_ext_hfosc());

        let usb_bus = UsbBusAllocator::new(Usbd::new(UsbPeripheral::new(
            periph.USBD,
            // refer to static lifetime
            cx.local.clocks.as_ref().unwrap(),
        )));
        cx.local.usb_bus.replace(usb_bus);

        let serial = SerialPort::new(&cx.local.usb_bus.as_ref().unwrap());

        let usb_dev = UsbDeviceBuilder::new(
            &cx.local.usb_bus.as_ref().unwrap(),
            UsbVidPid(0x16c0, 0x27dd),
        )
        .manufacturer("Fake company")
        .product("Serial port")
        .serial_number("TEST")
        .device_class(USB_CLASS_CDC)
        .max_packet_size_0(64) // (makes control transfers 8x faster)
        .build();

        #[allow(unreachable_code)]
        (Shared {}, Local {usb_dev,serial}, init::Monotonics())
    }


    #[task(binds = USBD,local =[usb_dev,serial,count:u32 = 0])]
    fn usb_interrupt(cx: usb_interrupt::Context) {

        let usb_interrupt::LocalResources { usb_dev, serial, count } = cx.local;
        let mut buf = [0u8;64];
        if !usb_dev.poll(&mut [serial]) {}

        match serial.read(&mut buf) {

            Ok(count) if count > 0 => {

                if let Some(index) = &buf.iter().position(|&x| x == 13) {
                    let sequence = &buf[0..*index];
                    match parse_result(&sequence) {
                        // Implement command parser
                        Ok(command_parser::Command::Stop) => {
                            rprintln!("stop");
                        },
                        Ok(command_parser::Command::Start) => {
                            rprintln!("start");
                        },
                        Ok(command_parser::Command::Duty(val)) => {
                            rprintln!("duty {:?}",val);
                        },
                        _ => {
                            for c in buf[0..count].iter_mut() {
                                if 0x61 <= *c && *c <= 0x7a {
                                    *c &= !0x20;
                                }
                            }
                            let mut write_offset = 0;
                            while write_offset < count {
                                match serial.write(&buf[write_offset..count]) {
                                    Ok(len) if len > 0 => {
                                        write_offset+=len;
                                    }, 
                                    _ => {}
                                }
                            }
                        },                                                      
                    };
                }
                
            },
            _ =>{},
        }
    }

}
