//! examples/rtc_usb_serial_interrupt
//!
//! CDC-ACM serial port example using polling in a busy loop.S
#![no_std]
#![no_main]

use panic_rtt_target as _;

#[rtic::app(device = nrf52840_hal::pac)]
mod app {

    use {
        hal::clocks::*,
        hal::usbd::{UsbPeripheral, Usbd},
        nrf52840_hal as hal,
        rtt_target::{rprintln, rtt_init_print},
        usb_device::{
            class_prelude::UsbBusAllocator,
            device::{UsbDevice, UsbDeviceBuilder, UsbVidPid},
        },
        usbd_serial::{SerialPort, USB_CLASS_CDC},
    };

    #[shared]
    struct Shared {}

    #[local]
    struct Local {
        usb_dev: UsbDevice<'static, Usbd<UsbPeripheral<'static>>>,
        serial: SerialPort<'static, Usbd<UsbPeripheral<'static>>>,
    }

    #[init(local = [
        clocks: Option<Clocks<ExternalOscillator, Internal, LfOscStopped>> = None,
        usb_bus: Option<UsbBusAllocator<Usbd<UsbPeripheral<'static>>>> = None, 
    ])]
    fn init(cx: init::Context) -> (Shared, Local, init::Monotonics) {
        rtt_init_print!();
        rprintln!("\n--- init ---");

        //Enable USBD interrupt
        cx.device.USBD.intenset.write(|w| w.sof().set());

        let periph = cx.device;
        let clocks = Clocks::new(periph.CLOCK);

        // make static lifetime for clocks
        cx.local.clocks.replace(clocks.enable_ext_hfosc());

        let usb_bus = UsbBusAllocator::new(Usbd::new(UsbPeripheral::new(
            periph.USBD,
            // refer to static lifetime
            cx.local.clocks.as_ref().unwrap(),
        )));
        cx.local.usb_bus.replace(usb_bus);

        let serial = SerialPort::new(&cx.local.usb_bus.as_ref().unwrap());

        let usb_dev = UsbDeviceBuilder::new(
            &cx.local.usb_bus.as_ref().unwrap(),
            UsbVidPid(0x16c0, 0x27dd),
        )
        .manufacturer("Fake company")
        .product("Serial port")
        .serial_number("TEST")
        .device_class(USB_CLASS_CDC)
        .max_packet_size_0(64) // (makes control transfers 8x faster)
        .build();

        (Shared {}, Local { usb_dev, serial }, init::Monotonics())
    }

    // interrupt generated by incoming usb traffic
    #[task(binds=USBD, local = [usb_dev, serial, count :u32 = 0])]
    fn usb_fs(cx: usb_fs::Context) {
        let usb_fs::LocalResources { usb_dev, serial, count } = cx.local;
        
        let mut buf = [0u8; 64];
        usb_dev.poll(&mut [serial]);
        if *count == 0 {
            rprintln!("usb_fs");
        }
        *count = (*count + 1) % 1000;

        match serial.read(&mut buf) {
            Ok(count) if count > 0 => {
                // Echo back in upper case
                for c in buf[0..count].iter_mut() {
                    if 0x61 <= *c && *c <= 0x7a {
                        *c &= !0x20;
                    }
                }

                let mut write_offset = 0;
                while write_offset < count {
                    match serial.write(&buf[write_offset..count]) {
                        Ok(len) if len > 0 => {
                            write_offset += len;
                        }
                        _ => {}
                    }
                }
            }
            _ => {}
        }
    }
}
